<?php

namespace ArsGeografica\Signing;


/**
 * Object signer.
 *
 * This class can generate a signature for any JSON-serializable object.
 *
 * @author Christian Wygoda
 */
class Signer
{
    protected $key;
    protected $salt;
    protected $sep;

    /**
     * Create a signer with a key
     *
     * @param  string  $key   Signing key to use
     */
    public function __construct($key, $sep=':', $salt=null)
    {
        $this->key = $key;
        $this->salt = (null !== $salt ? $salt : 'ArsGeograficaSigningSigner');
        $this->sep = $sep;
    }

    /**
     * Create a signed string (including the input value)
     *
     * @param   string  $string     String to be signed
     * @return  string  $signature
     */
    public function sign($value)
    {
        return sprintf('%s%s%s', $value, $this->sep, $this->signature($value));
    }

    /**
     * Test value signature. Raised BadSignatureException o fail.
     *
     * @param   string  $signed_value  Signature to check
     * @return  string  $value         Unsigned original value
     */
    public function unsign($signed_value)
    {
        if(false === strstr($signed_value, $this->sep)) {
            throw new BadSignatureException('No "' . $this->sep . '" in signed value.');
        }

        list($signature, $value) = array_map('strrev', explode($this->sep, strrev($signed_value), 2));

        if($this->signature($value) === $signature) {

            return $value;
        }

        throw new BadSignatureException('Signature "' . $signature . '" does not match value "' . $value . '"');
    }

    /**
     * Create a string signature (not including the input value)
     *
     * @param   string  $value      String to be signed
     * @return  string  $signature
     */
    public function signature($value)
    {
        $salt = $this->salt . 'signer';
        $key = sha1($salt . $this->key, true);

        $saltedHmac = hash_hmac('sha1', $value, $key, true);

        return rtrim(base64_encode($saltedHmac), '=');
    }
}
